import path from "path";
import globToRegExp from 'glob-to-regexp';
import { buildNormalizeFunctionForArray, buildNormalizeFunctionForObject, isSpecified, isStringAndNotEmpty, } from "./util/data.js";
const normalizeContentTypeDef = buildNormalizeFunctionForObject((config, errors) => {
    let { test, type, binary } = config;
    if (typeof test === 'function' || test instanceof RegExp) {
        // ok
    }
    else {
        errors.push('test cannot be null.');
    }
    if (isStringAndNotEmpty(type)) {
        // ok
    }
    else {
        errors.push('type must be a non-empty string.');
    }
    if (!isSpecified(config, 'binary')) {
        binary = false;
    }
    else {
        if (typeof binary === 'boolean') {
            // ok
        }
        else {
            errors.push('binary, if specified, must be a boolean value.');
        }
    }
    if (errors.length > 0) {
        return null;
    }
    return {
        test,
        contentType: type,
        binary,
    };
});
const normalizeContentTypeDefs = buildNormalizeFunctionForArray((config, errors) => {
    return normalizeContentTypeDef(config, errors);
});
const normalizePublisherServerConfig = buildNormalizeFunctionForObject((config, errors) => {
    let { publicDirPrefix, staticItems, compression, spaFile, notFoundPageFile, autoExt, autoIndex } = config;
    if (!isSpecified(config, 'publicDirPrefix')) {
        publicDirPrefix = '';
    }
    else {
        if (typeof publicDirPrefix === 'string') {
            // ok
        }
        else {
            errors.push('publicDirPrefix, if specified, must be a string value.');
        }
    }
    if (!isSpecified(config, 'staticItems')) {
        staticItems = [];
    }
    else {
        if (staticItems === null || staticItems === false) {
            staticItems = [];
        }
        if (!Array.isArray(staticItems)) {
            staticItems = [staticItems];
        }
        if (staticItems.every((x) => typeof x === 'string')) {
            staticItems = staticItems.map((x, index) => {
                if (x.includes('*')) {
                    let re;
                    try {
                        const regexp = globToRegExp(x, { globstar: true });
                        re = 're:' + String(regexp);
                    }
                    catch {
                        errors.push(`staticItems item at index ${index}, '${x}', cannot be parsed as glob pattern.`);
                        re = null;
                    }
                    return re;
                }
                return x;
            });
        }
        else {
            errors.push('staticItems, if specified, must be a string value, an array of string values, false, or null');
        }
    }
    if (!isSpecified(config, 'compression')) {
        compression = ['br', 'gzip'];
    }
    else if (compression === null) {
        compression = [];
    }
    else {
        if (!Array.isArray(compression)) {
            compression = [compression];
        }
        if (compression.some((x) => x !== 'br' && x !== 'gzip')) {
            errors.push(`compression, if specified, must be null or an array and can only contain 'br' and 'gzip'.`);
        }
    }
    if (!isSpecified(config, 'spaFile')) {
        spaFile = null;
    }
    else {
        if (typeof spaFile === 'string' || spaFile === null) {
            // ok
        }
        else if (spaFile === false) {
            spaFile = null;
        }
        else {
            errors.push('spaFile, if specified, must be a string value, false, or null');
        }
    }
    if (!isSpecified(config, 'notFoundPageFile')) {
        notFoundPageFile = null;
    }
    else {
        if (typeof notFoundPageFile === 'string' || notFoundPageFile === null) {
            // ok
        }
        else if (notFoundPageFile === false) {
            notFoundPageFile = null;
        }
        else {
            errors.push('notFoundPageFile, if specified, must be a string value, false, or null');
        }
    }
    if (!isSpecified(config, 'autoExt')) {
        autoExt = [];
    }
    else {
        if (Array.isArray(autoExt) && autoExt.every(e => typeof e === 'string')) {
            // ok
        }
        else if (autoExt === false || autoExt === null) {
            autoExt = [];
        }
        else if (typeof autoExt === 'string') {
            autoExt = [autoExt];
        }
        else {
            errors.push('autoExt, if specified, must be an array of string values, a string value, false, or null');
        }
    }
    if (!isSpecified(config, 'autoIndex')) {
        autoIndex = [];
    }
    else {
        if (Array.isArray(autoIndex) && autoIndex.every(e => typeof e === 'string')) {
            // ok
        }
        else if (autoIndex === false || autoIndex === null) {
            autoIndex = [];
        }
        else if (typeof autoIndex === 'string') {
            autoIndex = [autoIndex];
        }
        else {
            errors.push('autoIndex, if specified, must be an array of string values, a string value, false, or null');
        }
    }
    if (errors.length > 0) {
        return null;
    }
    return {
        publicDirPrefix,
        staticItems,
        compression,
        spaFile,
        notFoundPageFile,
        autoExt,
        autoIndex,
    };
});
const normalizeConfig = buildNormalizeFunctionForObject((config, errors) => {
    let { rootDir, objectStore, excludeDirs, excludeDotFiles, includeWellKnown, contentAssetInclusionTest, contentCompression, moduleAssetInclusionTest, contentTypes, server, } = config;
    if (!isSpecified(config, 'rootDir')) {
        errors.push('rootDir must be specified.');
    }
    else {
        if (isStringAndNotEmpty(rootDir)) {
            // ok
        }
        else {
            errors.push('rootDir must be a non-empty string.');
        }
    }
    if (!isSpecified(config, 'objectStore')) {
        objectStore = null;
    }
    else {
        if (isStringAndNotEmpty(objectStore) || objectStore === null) {
            // ok
        }
        else if (objectStore === false) {
            objectStore = null;
        }
        else {
            errors.push('objectStore, if specified, must be a non-empty string, false, or null.');
        }
    }
    if (!isSpecified(config, 'excludeDotFiles')) {
        excludeDotFiles = true;
    }
    else {
        if (typeof excludeDotFiles === 'boolean') {
            // ok
        }
        else {
            errors.push('excludeDotFiles, if specified, must be a boolean value.');
        }
    }
    if (!isSpecified(config, 'includeWellKnown')) {
        includeWellKnown = true;
    }
    else {
        if (typeof includeWellKnown === 'boolean') {
            // ok
        }
        else {
            errors.push('includeWellKnown, if specified, must be a boolean value.');
        }
    }
    if (!isSpecified(config, 'excludeDirs')) {
        excludeDirs = ['./node_modules'];
    }
    else {
        if (excludeDirs === null) {
            excludeDirs = [];
        }
    }
    if (!Array.isArray(excludeDirs)) {
        excludeDirs = [excludeDirs];
    }
    excludeDirs = excludeDirs.map((x) => {
        if (typeof x === 'string') {
            let testString = x;
            if (!testString.startsWith('/')) {
                if (testString.startsWith('./')) {
                    testString = testString.slice(1);
                }
                else {
                    testString = '/' + testString;
                }
            }
            if (testString.endsWith('/')) {
                testString = testString.slice(0, testString.length - 1);
            }
            return { test: (name) => name === testString, toString: () => testString };
        }
        if ((typeof x === 'object' || typeof x === 'function') && x != null) {
            if ('test' in x && typeof x.test === 'function') {
                return x;
            }
        }
        return null;
    });
    if (excludeDirs.some((x) => x == null)) {
        errors.push('excludeDirs, if specified, must be null, a string value, a RegExp, or an array of strings and RegExp values.');
    }
    if (!isSpecified(config, 'contentAssetInclusionTest')) {
        contentAssetInclusionTest = null;
    }
    else if (contentAssetInclusionTest === null || typeof contentAssetInclusionTest === 'function') {
        // ok
    }
    else {
        errors.push('contentAssetInclusionTest, if specified, must be null or a function.');
    }
    if (!isSpecified(config, 'contentCompression')) {
        contentCompression = ['br', 'gzip'];
    }
    else if (contentCompression === null) {
        contentCompression = [];
    }
    else {
        if (!Array.isArray(contentCompression)) {
            contentCompression = [contentCompression];
        }
        if (contentCompression.some((x) => x !== 'br' && x !== 'gzip')) {
            errors.push(`contentCompression, if specified, must be null or an array and can only contain 'br' and 'gzip'.`);
        }
    }
    if (!isSpecified(config, 'moduleAssetInclusionTest')) {
        moduleAssetInclusionTest = null;
    }
    else if (moduleAssetInclusionTest === null || typeof moduleAssetInclusionTest === 'function') {
        // ok
    }
    else {
        errors.push('moduleAssetInclusionTest, if specified, must be null or a function.');
    }
    if (!isSpecified(config, 'contentTypes')) {
        contentTypes = [];
    }
    else {
        const innerErrors = [];
        let normalized;
        if (Array.isArray(contentTypes) && (normalized = normalizeContentTypeDefs(contentTypes, innerErrors))) {
            // ok
            contentTypes = normalized;
        }
        else if (contentTypes === null) {
            contentTypes = [];
        }
        else {
            errors.push('contentTypes, if specified, must be null or an array of content type definitions.');
            errors.push(...innerErrors);
        }
    }
    if (!isSpecified(config, 'server')) {
        server = null;
    }
    else {
        const innerErrors = [];
        let normalized;
        if ((normalized = normalizePublisherServerConfig(server, innerErrors))) {
            // ok
            server = normalized;
        }
        else if (server === null) {
            // ok
        }
        else {
            errors.push('server, if specified, must be fit for server config.');
            errors.push(...innerErrors);
        }
    }
    if (errors.length > 0) {
        return null;
    }
    return {
        rootDir,
        objectStore,
        excludeDotFiles,
        includeWellKnown,
        excludeDirs,
        contentAssetInclusionTest,
        contentCompression,
        moduleAssetInclusionTest,
        contentTypes,
        server,
    };
});
export async function loadConfigFile(errors = []) {
    let config = undefined;
    const staticPublishRcPath = path.resolve('./static-publish.rc.js');
    try {
        config = (await import(staticPublishRcPath)).default;
    }
    catch {
        errors.push('Unable to load ' + staticPublishRcPath);
    }
    if (config != null) {
        config = normalizeConfig(config, errors);
    }
    return config ?? null;
}
