import { callFastlyApi } from "./fastly-api.js";
const cache = new WeakMap();
export async function getObjectStoreIdForNameMap(fastlyApiContext) {
    const cacheEntry = cache.get(fastlyApiContext);
    let objectStoreNameMap = cacheEntry?.objectStoreNameMap;
    if (objectStoreNameMap != null) {
        return objectStoreNameMap;
    }
    const objectStoreInfos = await getObjectStoreInfos(fastlyApiContext);
    objectStoreNameMap = {};
    for (const { id, name } of objectStoreInfos) {
        objectStoreNameMap[name] = id;
    }
    cache.set(fastlyApiContext, { ...cacheEntry, objectStoreNameMap });
    return objectStoreNameMap;
}
export async function getObjectStoreIdForName(fastlyApiContext, objectStoreName) {
    const objectStoreNameMap = await getObjectStoreIdForNameMap(fastlyApiContext);
    return objectStoreNameMap[objectStoreName] ?? null;
}
function createArrayGetter() {
    return function (fn) {
        return async function (fastlyApiContext, ...args) {
            const results = [];
            let cursor = null;
            while (true) {
                const queryParams = new URLSearchParams();
                if (cursor != null) {
                    queryParams.set('cursor', cursor);
                }
                const endpoint = fn(...args);
                const response = await callFastlyApi(fastlyApiContext, endpoint, queryParams);
                if (response.status !== 200) {
                    throw new Error('Unexpected data format');
                }
                const infos = await response.json();
                for (const item of infos.data) {
                    results.push(item);
                }
                if (infos.meta.next_cursor === undefined) {
                    break;
                }
                cursor = infos.meta.next_cursor;
            }
            return results;
        };
    };
}
const _getObjectStoreInfos = createArrayGetter()(() => `/resources/stores/object`);
export async function getObjectStoreInfos(fastlyApiContext) {
    const cacheEntry = cache.get(fastlyApiContext);
    let objectStoreInfos = cacheEntry?.objectStoreInfos;
    if (objectStoreInfos != null) {
        return objectStoreInfos;
    }
    objectStoreInfos = await _getObjectStoreInfos(fastlyApiContext);
    cache.set(fastlyApiContext, { ...cacheEntry, objectStoreInfos });
    return objectStoreInfos;
}
export const _getObjectStoreKeys = createArrayGetter()((objectStoreId) => `/resources/stores/object/${encodeURIComponent(objectStoreId)}/keys`);
export async function getObjectStoreKeys(fastlyApiContext, objectStoreName) {
    const objectStoreId = await getObjectStoreIdForName(fastlyApiContext, objectStoreName);
    if (objectStoreId == null) {
        return null;
    }
    return await _getObjectStoreKeys(fastlyApiContext, objectStoreId);
}
export async function objectStoreEntryExists(fastlyApiContext, objectStoreName, key) {
    const objectStoreId = await getObjectStoreIdForName(fastlyApiContext, objectStoreName);
    if (objectStoreId == null) {
        return false;
    }
    const endpoint = `/resources/stores/object/${encodeURIComponent(objectStoreId)}/keys/${encodeURIComponent(key)}`;
    const response = await callFastlyApi(fastlyApiContext, endpoint, null, { method: 'HEAD' });
    return response.status === 200;
}
const encoder = new TextEncoder();
export async function objectStoreSubmitFile(fastlyApiContext, objectStoreName, key, data) {
    const objectStoreId = await getObjectStoreIdForName(fastlyApiContext, objectStoreName);
    if (objectStoreId == null) {
        throw new Error('Object store not found');
    }
    const endpoint = `/resources/stores/object/${encodeURIComponent(objectStoreId)}/keys/${encodeURIComponent(key)}`;
    const body = typeof data === 'string' ? encoder.encode(data) : data;
    const response = await callFastlyApi(fastlyApiContext, endpoint, null, {
        method: 'PUT',
        headers: {
            'content-type': 'application/octet-stream',
        },
        body,
    });
    if (response.status !== 200) {
        throw new Error(`Submitting item ${key} gave error: ${response.status}`);
    }
}
export async function objectStoreDeleteFile(fastlyApiContext, objectStoreName, key) {
    const objectStoreId = await getObjectStoreIdForName(fastlyApiContext, objectStoreName);
    if (objectStoreId == null) {
        throw new Error('Object store not found');
    }
    const endpoint = `/resources/stores/object/${encodeURIComponent(objectStoreId)}/keys/${encodeURIComponent(key)}`;
    const response = await callFastlyApi(fastlyApiContext, endpoint, null, {
        method: 'DELETE',
    });
    if (response.status < 200 || response.status >= 300) {
        throw new Error(`Deleting item '${key}' gave error: ${response.status}`);
    }
}
